import ij.*;
import ij.plugin.*;
import org.mozilla.javascript.*;

public class JavaScript extends PlugInInterpreter implements PlugIn, Runnable {
	Thread thread;
	String script;
	String result;


	// run script on separate thread
	public void run(String script) {
		if (script.equals("")) return;
		this.script = script;
		thread = new Thread(this, "JavaScript"); 
		thread.setPriority(Math.max(thread.getPriority()-2, Thread.MIN_PRIORITY));
		thread.start();
	}
	
	// run script on current thread
	public String run(String script, String arg) {
		this.script = script;
		run();
		return null;
	}

	public String getReturnValue() {
		return result!=null?result:"";
	}

	public String getName() {
		return "JavaScript";
	}

	public String getVersion() {
		return "1.50g";
	}

	public String getImports() {
		return null;
	}

	public void run() {
		if (IJ.debugMode) IJ.log("Rhino: starting");
		result = "";
		Context cx = Context.enter();
		Scriptable scope = new ImporterTopLevel(cx);
		//Scriptable scope = cx.initStandardObjects(null);
		try {
			Object obj = cx.evaluateString(scope, script, "line", 1, null);                
			result = cx.toString(obj);
			if (IJ.debugMode) IJ.log("Rhino (normal return): "+result);
		} catch(Exception e) {
			if (IJ.debugMode) IJ.log("Rhino (exception): "+ e);
			result = ""+e;
			if (result.startsWith("org.mozilla.javascript.EcmaError: "))
				result = result.substring(34, result.length());
			if (result.indexOf("Macro canceled")!=-1)
				result = "";
			if (result.length()>0)
				IJ.log(result);
		} finally {
			Context.exit();
		}
	}

	public String toString() {
		return getReturnValue();
	}

}
